@@ -28,19 +28,28 @@ theme created via the dialog
 
 import gtk
 import gtk.glade
+import pango
 import os
 from ConfigParser import SafeConfigParser, NoOptionError
-from xdg.BaseDirectory import xdg_config_home, xdg_data_home
+from sys import platform
+if platform == 'win32':
+    data_home, config_home = (os.environ['APPDATA'],) * 2
+else:
+    from xdg.BaseDirectory import xdg_config_home as config_home
+    from xdg.BaseDirectory import xdg_data_home as data_home
 
+from gui import Theme
 from pyroom_error import PyroomError
 import autosave
 
 DEFAULT_CONF = {
     'visual':{
         'theme':'green',
-        'linenumber':'0',
         'showborder':'1',
         'linespacing':'2',
+        'custom_font':'Sans 12',
+        'use_font_type':'custom',
+        'indent':0,
     },
     'editor':{
         'session':'True',
@@ -83,18 +92,21 @@ class PyroomConfig(object):
 
     def __init__(self):
         self.pyroom_absolute_path = os.path.dirname(os.path.abspath(__file__))
-        self.conf_dir = os.path.join(xdg_config_home, 'pyroom')
-        self.data_dir = os.path.join(xdg_data_home, 'pyroom')
+        self.conf_dir = os.path.join(config_home, 'pyroom')
+        self.data_dir = os.path.join(data_home, 'pyroom')
         self.themes_dir  = os.path.join(self.data_dir, 'themes')
         self.global_themes_dir = '/usr/share/pyroom/themes'
         # if we are not using a global installation,
         # take the themes directly from sources
         if not os.path.isdir(self.global_themes_dir) :
-            self.global_themes_dir = os.path.join(
-                self.pyroom_absolute_path,
-                '..',
-                'themes'
-            )
+            if platform == 'win32':
+                self.global_themes_dir = ''
+            else:
+                self.global_themes_dir = os.path.join(
+                    self.pyroom_absolute_path,
+                    '..',
+                    'themes'
+                )
         self.conf_file = os.path.join(self.conf_dir, 'pyroom.conf')
         self.config = FailsafeConfigParser()
         self.build_default_conf()
@@ -139,16 +151,17 @@ class PyroomConfig(object):
 
 class Preferences(object):
     """our main preferences object, to be passed around where needed"""
-    def __init__(self, gui, style, pyroom_config):
-        self.style = style
+    def __init__(self, gui, pyroom_config):
         self.pyroom_config = pyroom_config
+        self.config = self.pyroom_config.config
+        self.graphical = gui
         self.wTree = gtk.glade.XML(os.path.join(
             pyroom_config.pyroom_absolute_path, "interface.glade"),
             "dialog-preferences")
 
+        self.gnome_fonts = self.get_gnome_fonts()
         # Defining widgets needed
         self.window = self.wTree.get_widget("dialog-preferences")
-        self.fontpreference = self.wTree.get_widget("fontbutton")
         self.colorpreference = self.wTree.get_widget("colorbutton")
         self.textboxbgpreference = self.wTree.get_widget("textboxbgbutton")
         self.bgpreference = self.wTree.get_widget("bgbutton")
@@ -159,13 +172,25 @@ class Preferences(object):
         self.widthpreference = self.wTree.get_widget("widthtext")
         self.widthpreference.set_range(5, 95)
         self.presetscombobox = self.wTree.get_widget("presetscombobox")
-        self.linenumbers = self.wTree.get_widget("linescheck")
         self.showborderbutton = self.wTree.get_widget("showborder")
         self.autosave = self.wTree.get_widget("autosavetext")
         self.autosave_spinbutton = self.wTree.get_widget("autosavetime")
         self.linespacing_spinbutton = self.wTree.get_widget("linespacing")
-
-        self.graphical = gui
+        self.indent_check = self.wTree.get_widget("indent_check")
+        if self.config.get('visual', 'indent') == '1':
+            self.indent_check.set_active(True)
+        self.save_custom_button = self.wTree.get_widget("save_custom_theme")
+        self.custom_font_preference = self.wTree.get_widget("fontbutton1")
+        if not self.config.get('visual', 'use_font_type') == 'custom':
+            self.custom_font_preference.set_sensitive(False)
+        self.font_radios = {
+            'document':self.wTree.get_widget("radio_document_font"),
+            'monospace':self.wTree.get_widget("radio_monospace_font"),
+            'custom':self.wTree.get_widget("radio_custom_font")
+        }
+        for widget in self.font_radios.values():
+            if not widget.get_name() == 'radio_custom_font':
+                widget.set_sensitive(bool(self.gnome_fonts))
 
         # Setting up config parser
         self.customfile = FailsafeConfigParser()
@@ -174,22 +199,18 @@ class Preferences(object):
         )
         if not self.customfile.has_section('theme'):
             self.customfile.add_section('theme')
-        self.config = self.pyroom_config.config
 
         # Getting preferences from conf file
         self.activestyle = self.config.get("visual", "theme")
-        self.linesstate = self.config.get("visual", "linenumber")
         self.pyroom_config.showborderstate = self.config.get(
             "visual", "showborder"
         )
         self.autosavestate = self.config.get("editor", "autosave")
-        self.autosavetime = self.config.get("editor", "autosavetime")
-        self.linespacing = self.config.get("visual", "linespacing")
-        if self.autosavestate == 1:
-            autosave.autosave_time = self.autosavetime
+        if int(self.autosavestate) == 1:
+            self.autosave_time = self.config.get("editor", "autosavetime")
         else:
-            autosave.autosave_time = 0
-        self.linesstate = int(self.linesstate)
+            self.autosave_time = 0
+        self.linespacing = self.config.get("visual", "linespacing")
         self.pyroom_config.showborderstate = int(
             self.pyroom_config.showborderstate
         )
@@ -197,26 +218,32 @@ class Preferences(object):
 
         # Set up pyroom from conf file
         self.linespacing_spinbutton.set_value(int(self.linespacing))
-        self.autosave_spinbutton.set_value(float(self.autosavetime))
-        self.linenumbers.set_active(self.linesstate)
+        self.autosave_spinbutton.set_value(float(self.autosave_time))
         self.autosave.set_active(self.autosavestate)
         self.showborderbutton.set_active(self.pyroom_config.showborderstate)
+        font_type = self.config.get('visual', 'use_font_type')
+        self.font_radios[font_type].set_active(True)
+        
         self.toggleautosave(self.autosave)
 
-        
         self.window.set_transient_for(self.graphical.window)
 
         self.stylesvalues = {'custom': 0}
         self.startingvalue = 1
 
+
+        self.graphical.theme = Theme(self.config.get('visual', 'theme'))
         # Add themes to combobox
         for i in self.pyroom_config.themeslist:
             self.stylesvalues['%s' % (i)] = self.startingvalue
             self.startingvalue = self.startingvalue + 1
-            i = i.capitalize()
-            self.presetscombobox.append_text(i)
+            current_loading_theme = Theme(i)
+            theme_name = current_loading_theme['name']
+            self.presetscombobox.append_text(theme_name)
+        if self.activestyle == 'custom':
+            self.save_custom_button.set_sensitive(True)
         self.presetscombobox.set_active(self.stylesvalues[self.activestyle])
-        self.presetchanged(self.presetscombobox, 'initial')
+        self.fill_pref_dialog()
 
         # Connecting interface's signals
         dic = {
@@ -226,15 +253,16 @@ class Preferences(object):
                 "on_close": self.kill_preferences
                 }
         self.wTree.signal_autoconnect(dic)
-        self.linenumbers.connect('toggled', self.togglelines)
         self.showborderbutton.connect('toggled', self.toggleborder)
         self.autosave.connect('toggled', self.toggleautosave)
         self.autosave_spinbutton.connect('value-changed', self.toggleautosave)
         self.linespacing_spinbutton.connect(
             'value-changed', self.changelinespacing
         )
+        self.indent_check.connect(
+            'toggled', self.toggle_indent
+        )
         self.presetscombobox.connect('changed', self.presetchanged)
-        self.fontpreference.connect('font-set', self.customchanged)
         self.colorpreference.connect('color-set', self.customchanged)
         self.textboxbgpreference.connect('color-set', self.customchanged)
         self.bgpreference.connect('color-set', self.customchanged)
@@ -242,12 +270,56 @@ class Preferences(object):
         self.paddingpreference.connect('value-changed', self.customchanged)
         self.heightpreference.connect('value-changed', self.customchanged)
         self.widthpreference.connect('value-changed', self.customchanged)
+        self.save_custom_button.connect('clicked', self.save_custom_theme)
+        for widget in self.font_radios.values():
+            widget.connect('toggled', self.change_font)
+        self.custom_font_preference.connect('font-set', self.change_font)
+        self.set_font()
 
+    def get_gnome_fonts(self):
+        """test if gnome font settings exist"""
+        try:
+            import gconf
+        except ImportError:
+            return
+        gconf_client = gconf.Client()
+        fonts = {'document':'', 'monospace':''}
+        try:
+            for font in fonts.keys():
+                fonts[font] = gconf_client.get_value(
+                    '/desktop/gnome/interface/%s_font_name' % 
+                    font
+                )
+        except ValueError:
+            return
+        else:
+            return fonts
+
+    def change_font(self, widget):
+        if widget.get_name() in ('fontbutton1', 'radio_custom_font'):
+            self.custom_font_preference.set_sensitive(True)
+            new_font = self.custom_font_preference.get_font_name()
+            self.config.set('visual', 'use_font_type', 'custom')
+            self.config.set('visual', 'custom_font', new_font)
+        else:
+            self.custom_font_preference.set_sensitive(False)
+            font_type = widget.get_name().split('_')[1]
+            self.config.set('visual', 'use_font_type', font_type)
+        self.set_font()
+        self.graphical.apply_theme()
+    
+    def set_font(self):
+        """set font according to settings"""
+        if self.config.get('visual', 'use_font_type') == 'custom' or\
+           not self.gnome_fonts:
+            new_font = self.config.get('visual', 'custom_font')
+        else:
+            font_type = self.config.get('visual', 'use_font_type')
+            new_font = self.gnome_fonts[font_type]
+        self.graphical.textbox.modify_font(pango.FontDescription(new_font))
+        
     def getcustomdata(self):
         """reads custom themes"""
-        self.fontname = self.fontpreference.get_font_name()
-        self.fontsize = int(self.fontname[-2:])
-        self.fontname = self.fontname[:-2]
         self.colorname = gtk.gdk.Color.to_string(
                                 self.colorpreference.get_color())
         self.textboxbgname = gtk.gdk.Color.to_string(
@@ -260,27 +332,42 @@ class Preferences(object):
         self.heightname = self.heightpreference.get_value() / 100.0
         self.widthname = self.widthpreference.get_value() / 100.0
 
+    def save_custom_theme(self, widget, data=None):
+        chooser = gtk.FileChooserDialog('PyRoom', self.window, 
+            gtk.FILE_CHOOSER_ACTION_SAVE,
+            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+            gtk.STOCK_SAVE, gtk.RESPONSE_OK)
+        )
+        chooser.set_default_response(gtk.RESPONSE_OK)
+        chooser.set_current_folder(self.pyroom_config.themes_dir)
+        filter_pattern = gtk.FileFilter()
+        filter_pattern.add_pattern('*.theme')
+        filter_pattern.set_name(_('Theme Files'))
+        chooser.add_filter(filter_pattern)
+        res = chooser.run()
+        if res == gtk.RESPONSE_OK:
+            theme_filename = chooser.get_filename()
+            self.graphical.theme.save(theme_filename)
+            theme_name = os.path.basename(theme_filename)
+            theme_id = max(self.stylesvalues.values()) + 1
+            self.presetscombobox.append_text(theme_name)
+            self.stylesvalues.update({theme_name: theme_id})
+            self.presetscombobox.set_active(theme_id)
+        chooser.destroy()
+
     def set_preferences(self, widget, data=None):
         """save preferences"""
         self.getcustomdata()
-        self.linenumberspref = self.linenumbers.get_active()
         self.autosavepref = self.autosave.get_active()
-        if self.linenumberspref == True:
-            self.linenumberspref = 1
-        else:
-            self.linenumberspref = 0
         if self.autosavepref == True:
             self.autosavepref = 1
         else:
             self.autosavepref = 0
-        self.config.set("visual", "showborder", 
-                        str(self.pyroom_config.showborderstate))
-        self.config.set("visual", "linenumber", str(self.linenumberspref))
         self.config.set("editor", "autosave", str(self.autosavepref))
         self.config.set("visual", "linespacing", str(int(self.linespacing)))
 
-        autosave.autosave_time = self.autosave_spinbutton.get_value_as_int()
-        self.config.set("editor", "autosavetime", str(autosave.autosave_time))
+        autosave_time = self.autosave_spinbutton.get_value_as_int()
+        self.config.set("editor", "autosavetime", str(autosave_time))
 
         if self.presetscombobox.get_active_text().lower() == 'custom':
             custom_theme = open(os.path.join(
@@ -291,8 +378,6 @@ class Preferences(object):
             self.customfile.set("theme", "foreground", self.colorname)
             self.customfile.set("theme", "textboxbg", self.textboxbgname)
             self.customfile.set("theme", "border", self.bordername)
-            self.customfile.set("theme", "font", self.fontname)
-            self.customfile.set("theme", "fontsize", str(self.fontsize))
             self.customfile.set("theme", "padding", str(self.paddingname))
             self.customfile.set("theme", "width", str(self.widthname))
             self.customfile.set("theme", "height", str(self.heightname))
@@ -310,96 +395,101 @@ class Preferences(object):
         self.presetscombobox.set_active(0)
         self.presetchanged(widget)
 
+    def fill_pref_dialog(self):
+        self.custom_font_preference.set_font_name(
+            self.config.get('visual', 'custom_font')
+        )
+        parse_color = lambda x: gtk.gdk.color_parse(
+            self.graphical.theme[x]
+        )
+        self.colorpreference.set_color(parse_color('foreground'))
+        self.textboxbgpreference.set_color(parse_color('textboxbg'))
+        self.bgpreference.set_color(parse_color('background'))
+        self.borderpreference.set_color(parse_color('border'))
+        self.paddingpreference.set_value(float(
+            self.graphical.theme['padding'])
+        )
+        self.widthpreference.set_value(
+            float(self.graphical.theme['width']) * 100
+        )
+        self.heightpreference.set_value(
+            float(self.graphical.theme['height']) * 100
+        )
+
     def presetchanged(self, widget, mode=None):
         """some presets have changed, apply those"""
-        if mode == 'initial':
-            self.fontname = "%s %s" % (
-                self.graphical.config.get("theme", "font"),
-                self.graphical.config.get("theme", "fontsize")
-            )
-            self.fontpreference.set_font_name(self.fontname)
-            self.colorpreference.set_color(gtk.gdk.color_parse(
-             self.graphical.config.get("theme", "foreground")))
-            self.textboxbgpreference.set_color(gtk.gdk.color_parse(
-             self.graphical.config.get("theme", "textboxbg")))
-            self.bgpreference.set_color(gtk.gdk.color_parse(
-             self.graphical.config.get("theme", "background")))
-            self.borderpreference.set_color(gtk.gdk.color_parse(
-                  self.graphical.config.get("theme", "border")))
-            self.paddingpreference.set_value(float(self.graphical.config.get(
-                                                        "theme", "padding")))
-            self.widthpreference.set_value(float(self.graphical.config.get(
-                                                       "theme", "width")) * 100)
-            self.heightpreference.set_value(float(self.graphical.config.get(
-                                                       "theme", "height")) *100)
-            self.graphical.apply_style(self.style, 'normal')
+        active_theme_id = self.presetscombobox.get_active()
+        for key, value in self.stylesvalues.iteritems():
+            if value == active_theme_id:
+                active_theme = key
+        if active_theme_id == 0:
+            self.getcustomdata()
+            custom_theme = Theme('custom')
+            custom_theme.update({
+                    'name': 'custom',
+                    'background': self.bgname,
+                    'textboxbg': self.textboxbgname,
+                    'foreground': self.colorname,
+                    'lines': self.colorname,
+                    'border': self.bordername,
+                    'info': self.colorname,
+                    'padding': self.paddingname,
+                    'height': self.heightname,
+                    'width': self.widthname,
+            })
+
+            self.config.set("visual", "theme", str(active_theme))
+            self.graphical.theme = custom_theme
+            self.save_custom_button.set_sensitive(True)
         else:
-            active = self.presetscombobox.get_active_text().lower()
-            activeid = self.presetscombobox.get_active()
-            if activeid == 0:
-                self.getcustomdata()
-                customstyle = {
-                    'Custom': {
-                        'name': 'custom',
-                        'background': self.bgname,
-                        'textboxbg': self.textboxbgname,
-                        'foreground': self.colorname,
-                        'lines': self.colorname,
-                        'border': self.bordername,
-                        'info': self.colorname,
-                        'font': self.fontname,
-                        'fontsize': self.fontsize,
-                        'padding': self.paddingname,
-                        'size': [self.widthname, self.heightname],
-                }}
-                self.graphical.apply_style(customstyle['Custom'], 'custom')
-                self.config.set("visual", "theme", str(active))
-                self.graphical.status.set_text(_('Style Changed to \
-                                                %s' % (active)))
-            else:
-                theme = os.path.join(self.pyroom_config.themes_dir,
-                    active + ".theme")
-                #if theme doesn't exist in personnal config
-                # we take it in the global theme directory
-                if not os.path.isfile(theme) :
-                	theme = os.path.join(self.pyroom_config.global_themes_dir,
-                		active + ".theme")          
-                self.graphical.config.read(theme)
-                self.graphical.apply_style()
-                self.fontname = "%s %s" % (
-                    self.graphical.config.get("theme", "font"),
-                    self.graphical.config.get("theme", "fontsize"))
-                self.fontpreference.set_font_name(self.fontname)
-                self.config.set("visual", "theme", str(active))
-                self.colorpreference.set_color(gtk.gdk.color_parse(
-                 self.graphical.config.get("theme", "foreground")))
-                self.bgpreference.set_color(gtk.gdk.color_parse(
-                 self.graphical.config.get("theme", "background")))
-                self.textboxbgpreference.set_color(gtk.gdk.color_parse(
-                 self.graphical.config.get("theme", "textboxbg")))
-                self.borderpreference.set_color(gtk.gdk.color_parse(
-                      self.graphical.config.get("theme", "border")))
-                self.paddingpreference.set_value(float(
-                 self.graphical.config.get("theme", "padding")))
-                self.widthpreference.set_value(float(self.graphical.config.get(
-                                                     "theme", "width")) * 100)
-                self.heightpreference.set_value(float(
-                 self.graphical.config.get("theme", "height")) * 100)
-                self.graphical.status.set_text(_('Style Changed to \
-%s' % (active)))
-                self.presetscombobox.set_active(activeid)
+            new_theme = Theme(active_theme)
+            self.graphical.theme = new_theme
+            parse_color = lambda x: gtk.gdk.color_parse(
+                self.graphical.theme[x]
+            )
+            self.colorpreference.set_color(parse_color('foreground'))
+            self.textboxbgpreference.set_color(parse_color('textboxbg'))
+            self.bgpreference.set_color(parse_color('background'))
+            self.borderpreference.set_color(parse_color('border'))
+            self.paddingpreference.set_value(float(
+                self.graphical.theme['padding'])
+            )
+            self.widthpreference.set_value(
+                float(self.graphical.theme['width']) * 100
+            )
+            self.heightpreference.set_value(
+                float(self.graphical.theme['height']) * 100
+            )
+            self.config.set("visual", "theme", str(active_theme))
+            self.presetscombobox.set_active(active_theme_id)
+            self.save_custom_button.set_sensitive(False)
+
+        self.graphical.apply_theme()
+        self.graphical.status.set_text(_('Style Changed to %s') %
+                                        (active_theme))
 
     def show(self):
         """display the preferences dialog"""
         self.dlg = self.wTree.get_widget("dialog-preferences")
         self.dlg.show()
 
+    def toggle_indent(self, widget):
+        """toggle textbox indent"""
+        if self.config.get('visual', 'indent') == '1':
+            self.config.set('visual', 'indent', '0')
+        else:
+            self.config.set('visual', 'indent', '1')
+        self.graphical.apply_theme()
+
     def toggleborder(self, widget):
         """toggle border display"""
+        #FIXME just workaround, we should drop pyroom_config entirely
         if self.pyroom_config.showborderstate:
             self.pyroom_config.showborderstate = 0
+            self.config.set('visual', 'showborder', '0')
         else:
             self.pyroom_config.showborderstate = 1
+            self.config.set('visual', 'showborder', '1')
         self.graphical.boxout.set_border_width(
             self.pyroom_config.showborderstate
         )
@@ -407,11 +497,6 @@ class Preferences(object):
             self.pyroom_config.showborderstate
         )
 
-    def togglelines(self, widget):
-        """show line numbers"""
-        opposite_state = not self.graphical.textbox.get_show_line_numbers()
-        self.graphical.textbox.set_show_line_numbers(opposite_state)
-
     def changelinespacing(self, widget):
         """Change line spacing"""
         self.linespacing = self.linespacing_spinbutton.get_value()
@@ -424,10 +509,10 @@ class Preferences(object):
         """enable or disable autosave"""
         if self.autosave.get_active():
             self.autosave_spinbutton.set_sensitive(True)
-            autosave.autosave_time = self.autosave_spinbutton.get_value_as_int()
+            self.autosave_time = self.autosave_spinbutton.get_value_as_int()
         else:
             self.autosave_spinbutton.set_sensitive(False)
-            autosave.autosave_time = 0
+            self.autosave_time = 0
 
     def QuitEvent(self, widget, data=None):
         """quit our app"""
